// www.sgm.siedlce.pl

// dekodowanie sygnalu C-PPM (PPM-Sum)
// test uzytkowy w locie modelem DWS (latajace skrzydlo)
// prosty filtr usredniajacy
// tylko kanaly 1,2,3 (gaz i 2 serwa sterolotek)
// wynki odczytu w zmiennej Chan[]
// Chan[0] impuls synchronizujacy
// Chan[1] kanal 1
// ...
// Chan[8] kanal 8

#include <Servo.h>  // dołączenie biblioteki
Servo servoCh1;      // utworzenie obiektu 
Servo servoCh2;      // utworzenie obiektu
Servo servoCh3;      // utworzenie obiektu

/*  pomiar czasu (micros) za pomocą biblioteki
===================================================================================================
<eRCaGuy_Timer2_Counter.h> by Gabriel Staples
Visit my blog at http://electricrcaircraftguy.blogspot.com/
-My contact info is available by clicking the "Contact Me" tab at the top of my blog.
-Please support my work & contributions by buying something here: https://sites.google.com/site/ercaguystore1/
My original post containing this code can be found here: http://electricrcaircraftguy.blogspot.com/2014/02/Timer2Counter-more-precise-Arduino-micros-function.html
Written: 17 May 2014
Updated: 30 May 2014
===================================================================================================
  LICENSE & DISCLAIMER
  Copyright (C) 2015 Gabriel Staples.  All right reserved.
  
  ------------------------------------------------------------------------------------------------
  License: GNU Lesser General Public License Version 3 (LGPLv3) or later - https://www.gnu.org/licenses/lgpl.html
  ------------------------------------------------------------------------------------------------

  This file is part of eRCaGuy_Timer2_Counter.
  
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program.  If not, see http://www.gnu.org/licenses/
===================================================================================================
*/

#include <eRCaGuy_Timer2_Counter.h>
// funkcja timer2.get_count()  podaje wynik w jednostkach 0.5us
// funkcja timer2.get_micros() podaje wynik w mikrosekundach
 
const int butIrq = 1;     // Irq 1 na Pin2 (wejscie)
 
volatile int pulse_t1 = 0;  
volatile int pulse_t2 = 0;  
volatile int pulse_t3 = 0;  

int t1_t2_micros = 0;                                  
int t1_t3_micros = 0;
long m_timer=0;         
long gaptimer2 = 330;  

//obsluga pomiaru - testowo
float valPulse;           // zmienna na dlugosc 'Pulse' (microsec)
float valPeriod;          // zmienna na dlugosc 'Period'

//poczatkowe wartosci kanalow, Chan1 - gaz
int Chan[] = {4000,1100,1500,1500,1500,1500,1500,1500,1500};
byte ChNum;
// pomiar przez bufor i usrednianie
int ChanB1[] = {2200,1100,1100}; //bufor kan 1 na 2 pomiary i sume [0]
int ChanB2[] = {3000,1500,1500}; //bufor kan 2 na 2 pomiary i sume [0]
int ChanB3[] = {3000,1500,1500}; //bufor kan 3 na 2 pomiary i sume [0]
const int dP = 24;  // 'znieczulenie' -  minimalna zmiana wartosci impulsu


//*****************************************program 
void setup() {
 //obowiazkowa inicjalizacja Timer2 wymagana przez
 //biblioteke <eRCaGuy_Timer2_Counter.h>
 timer2.setup();
 
 servoCh1.attach(9);
 servoCh1.writeMicroseconds(1100);
 servoCh2.attach(6);
 servoCh2.writeMicroseconds(1500);
 servoCh3.attach(5);
 servoCh3.writeMicroseconds(1500);
 
// Serial.begin(115200);  //w czasie uruchamiania
 
 attachInterrupt(0, pulse_rising, RISING);
 
 delay(3000);
 
}
 
void loop() {
  
 //przesylanie zmierzonych wartosci do serw w procedurze
 //obslugi przerwania void pulse_rising()
  
 //testowe wyswietlanie co gaptimer2
 if (millis()- m_timer > gaptimer2) {
   m_timer=millis();
   
/*   
   Serial.print("0:"); Serial.print(Chan[0]); Serial.print("; ");
   Serial.print("1B:"); Serial.print(ChanB1[2]); Serial.print("; ");
   Serial.print("1:"); Serial.print(Chan[1]); Serial.print("; ");
   Serial.print("2:"); Serial.print(Chan[2]); Serial.print("; ");
   Serial.print("3:"); Serial.print(Chan[3]); Serial.println(); //Serial.print("; ");
*/
  
 }

} // void loop()



// obsluga przerwania 0 (rising) //poczatek impulsu
void pulse_rising() {
  attachInterrupt(0, pulse_falling, FALLING);
  pulse_t3 = pulse_t1;
  pulse_t1 = timer2.get_micros();
  t1_t3_micros = pulse_t1 - pulse_t3; //period
   
  if (ChNum < 9) {
    Chan[ChNum] = t1_t3_micros;
    //Chan[ChNum] = ((t1_t3_micros + 5)/10)*10; //zaokraglenie do '10'
    
    if (ChNum == 1) {                    //obrobka dla kan1
     ChanB1[0] = ChanB1[1] + ChanB1[2];  //suma do zmiennej [0]
     ChanB1[2] = ChanB1[1];              //przesun [1] --> [2]
     if ((ChanB1[1] - t1_t3_micros) <dP || (ChanB1[1] - t1_t3_micros)>dP) {
      ChanB1[1] = (ChanB1[0] + t1_t3_micros)/3;     //odrzucanie za malego wzrostu
     }
     servoCh1.writeMicroseconds(ChanB1[2]); // [2] z bufora-filtra, FIFO 
    }
    
    if (ChNum == 2) {                    //obrobka dla kan2
     ChanB2[0] = ChanB2[1] + ChanB2[2];  //suma do zmiennej [0]
     ChanB2[2] = ChanB2[1];              //przesun [1] --> [2]
     if ((ChanB2[1] - t1_t3_micros) <dP || (ChanB2[1] - t1_t3_micros)>dP) {
      ChanB2[1] = (ChanB2[0] + t1_t3_micros)/3;     //odrzucanie za malego wzrostu
     }
     servoCh2.writeMicroseconds(ChanB2[2]); // [2] z bufora-filtra, FIFO 
    }

    if (ChNum == 3) {                    //obrobka dla kan3
     ChanB3[0] = ChanB3[1] + ChanB3[2];  //suma do zmiennej [0]
     ChanB3[2] = ChanB3[1];              //przesun [1] --> [2]
     if ((ChanB3[1] - t1_t3_micros) <dP || (ChanB3[1] - t1_t3_micros)>dP) {
      ChanB3[1] = (ChanB3[0] + t1_t3_micros)/3;     //odrzucanie za malego wzrostu
     }
     servoCh3.writeMicroseconds(ChanB3[2]); // [2] z bufora-filtra, FIFO 
    }    
    
    ChNum ++;
  }   
 
} //void pulse_rising()


// obsluga przerwania 0 (falling)  //koniec impulsu
void pulse_falling() {
  attachInterrupt(0, pulse_rising, RISING);
  pulse_t2 = timer2.get_micros();
  t1_t2_micros = pulse_t2 - pulse_t1;
  
  if (t1_t2_micros > 3000) {
   ChNum = 0; 
  }  
  
} //void pulse_falling()
